﻿-- set to true for additional dev messages
local BMTDebug = true;
-- title text on top of the browser window
local BMT_TITLE = "Black Market Tracker";
-- new addon version text shown at the bottom of the browser
local BMT_NEW_VERSION = "New addon version available";
-- coming soon text
local BMT_COMING_SOON = "Coming Soon!";
-- alpa for fading bottom text, goes above 1.0 so that the text stays lit for a while
local BMT_TEXT_ALPHA = 6.0;
-- time of fade in/out
local BMT_TEXT_TIME = 2.5;
-- sync string separator
local BMT_SEPARATOR = "%";
-- sync sender key
local BMT_SYNC_SENDER_KEY = "SENDER";
-- sync item key
local BMT_SYNC_ITEM_KEY = "ITEM";
-- sync version key
local BMT_SYNC_VER_KEY = "VER";
-- predicted time left constants (time in minutes) - completed, short, medium, long, very long
local BMT_TIME_LEFT = {0, 30, 120, 720, 9999};
-- this revision number
local BMTRevision = 6;
-- highest revision gotten from others upon logging in
BMTHighestRevision = BMTRevision;
-- fading state (bottom text)
BMTFadeState = 0;
-- last updated time string, used to throttle the sync results (when multiple events get fired after opening BMAH window once)
BMTLastUpdateTime = "-";
-- TODO: Temp stuff, remove
BMTPredictedLow = "99:99";
BMTPredictedHigh = "99:99";

function BMT_Debug(msg)
	if(BMTDebug) then
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

function BMT_SlashDebugHandler()
	BMT_CompareDate("25/09/12");
end

function BMT_SlashHandler(msg, editbox)
	local msgLen = #msg;
	-- TODO: Config
	if(msgLen == 0 or msg == "show" or msg == "ui" or msg == "open") then
		BMTFrame_Toggle();
	elseif(msg == "debug") then
		BMT_SlashDebugHandler();
	end
end

function BMTFrame_OnLoad(self)
	SLASH_BLACKMARKETTRACKER1 = "/bmt";
	SLASH_BLACKMARKETTRACKER2 = "/blackmarkettracker";
	SlashCmdList["BLACKMARKETTRACKER"] = BMT_SlashHandler;
	self:RegisterForDrag("LeftButton");
	tinsert(UISpecialFrames, self:GetName());
	self:RegisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("CHAT_MSG_ADDON");
	self:RegisterEvent("BLACK_MARKET_ITEM_UPDATE");
	BlackMarketTrackerScrollFrame.scrollBar.doNotHide = true;
	BlackMarketTrackerFrameTitleText:SetText(BMT_TITLE);
	RegisterAddonMessagePrefix("BMT");
end

function BMTFrame_OnEvent(self, event, ...)
	if(event == "BLACK_MARKET_ITEM_UPDATE") then
		BMT_NewSearchHandler();
	elseif(event == "CHAT_MSG_ADDON") then
		local prefix = select(1, ...);
		local msg = select(2, ...);
		local channel = select(3, ...);
		local sender = select(4, ...);
		BMT_AddonMsgReceiveHandler(prefix, msg, channel, sender);
	elseif(event == "PLAYER_LOGIN") then
		BMT_OnLoginHandler();
	end
end

function BMTFrame_Toggle()
	if(BlackMarketTrackerFrame:IsShown()) then
		BlackMarketTrackerFrame:Hide();
		--BlackMarketTrackerBottom:SetAlpha(0);
		-- TODO: sound enable toggle
	else
		BlackMarketTrackerFrame:Show();
		-- TODO: sound enable toggle
	end
end

function BMTFrame_OnShow()
	if(BMTHighestRevision <= BMTRevision) then
		BMTFadeState = 2;
	else
		BMTFadeState = 0;
	end
	PlaySound("AuctionWindowOpen");
	HybridScrollFrame_CreateButtons(BlackMarketTrackerScrollFrame, "BlackMarketTrackerItemTemplate", 5, -5);
	BlackMarketTrackerScrollFrame_Update();
	BlackMarketTrackerBottomText_FadingHandler();
end

function BlackMarketTrackerBottomText_FadingHandler()
	if(BMTFadeState == 0) then
		BMTFadeState = 1;
		BlackMarketTrackerFrameBottomText:SetText(BMT_NEW_VERSION .. " (r" .. BMTHighestRevision .. ")");
		BlackMarketTrackerBottomText_FadeIn();
	elseif(BMTFadeState == 1) then
		BMTFadeState = 2;
		BlackMarketTrackerBottomText_FadeOut();
	elseif(BMTFadeState == 2) then
		BMTFadeState = 3;
		local bottomText = "Last updated by ";
		if(BlackMarketTrackerMisc) then
			if(not BlackMarketTrackerMisc.UpdaterName or not BlackMarketTrackerMisc.UpdaterClass or not BlackMarketTrackerMisc.UpdaterGuild or not BlackMarketTrackerMisc.UpdateTime or not BlackMarketTrackerMisc.UpdateDate) then
				bottomText = bottomText .. "|cffffd100Unknown|r";
			else
				local classColor = RAID_CLASS_COLORS[BlackMarketTrackerMisc.UpdaterClass];
				bottomText = bottomText .. "|cff" .. string.format("%2.2x", classColor.r*255) .. string.format("%2.2x", classColor.g*255) .. string.format("%2.2x", classColor.b*255);
				bottomText = bottomText .. BlackMarketTrackerMisc.UpdaterName .. "|r (";
				bottomText = bottomText .. BlackMarketTrackerMisc.UpdaterGuild .. "), ";
				local updateTimeStr =  string.format("%2.2d:%2.2d", (BlackMarketTrackerMisc.UpdateTime/60), (BlackMarketTrackerMisc.UpdateTime%60));
				bottomText = bottomText .. BlackMarketTrackerMisc.UpdateDate .. " " .. updateTimeStr;
			end
		else
			bottomText = bottomText .. "|cffffd100Unknown|r";
		end
		BlackMarketTrackerFrameBottomText:SetText(bottomText);
		BlackMarketTrackerBottomText_FadeIn();
	end
end

function BlackMarketTrackerBottomText_FadeIn()
	local fadeInfo = {};
    fadeInfo.mode = "IN";
    fadeInfo.timeToFade = BMT_TEXT_TIME;
	fadeInfo.endAlpha = BMT_TEXT_ALPHA;
    fadeInfo.finishedFunc = BlackMarketTrackerBottomText_FadingHandler;
    UIFrameFade(BlackMarketTrackerFrameBottom, fadeInfo);
end

function BlackMarketTrackerBottomText_FadeOut()
	local fadeInfo = {};
    fadeInfo.mode = "OUT";
    fadeInfo.timeToFade = BMT_TEXT_TIME;
	fadeInfo.startAlpha = BMT_TEXT_ALPHA;
    fadeInfo.finishedFunc = BlackMarketTrackerBottomText_FadingHandler;
    UIFrameFade(BlackMarketTrackerFrameBottom, fadeInfo);
end

function BlackMarketTrackerScrollFrame_Update()
	local numItems = 0;
	if(BlackMarketTrackerCurrentDB ~= nil) then 
		numItems = #BlackMarketTrackerCurrentDB;
	end
	
	local scrollFrame = BlackMarketTrackerScrollFrame;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons = scrollFrame.buttons;
	local numButtons = #buttons;
	
	for i=1, numButtons do
		local button = buttons[i];
		-- TODO: (should be history db as well) get index of an item in black market
		local index = offset + i;
		if ( index <= numItems ) then
			local name, texture, quantity, itemType, level, levelType, sellerName, minBid, minIncrement, currBid, youHaveHighBid, numBids, timeLeft, link, marketID = BMT_GetFromDB(BlackMarketTrackerCurrentDB, index);
			if(name) then
				button.Name:SetText(name);
                button.Item.IconTexture:SetTexture(texture);
				--[[
                if ( not usable ) then
                    button.Item.IconTexture:SetVertexColor(1.0, 0.1, 0.1);
                else
                    button.Item.IconTexture:SetVertexColor(1.0, 1.0, 1.0);
                end
				]]
                button.Item.Count:SetText(quantity);
                button.Item.Count:SetShown(quantity > 1);
                button.Type:SetText(itemType);
                button.Seller:SetText(sellerName);
                button.Level:SetText(level);
                local bidAmount = currBid;
                local minNextBid = currBid + minIncrement;
                if ( currBid == 0 ) then
                    bidAmount = minBid;
                    minNextBid = minBid;
                end
                MoneyFrame_Update(button.CurrentBid, bidAmount);
                button.minNextBid = minNextBid;
                button.auctionCompleate = (timeLeft == 0);
                button.TimeLeft.Text:SetText(_G["AUCTION_TIME_LEFT"..timeLeft]);
                button.TimeLeft.tooltip = _G["AUCTION_TIME_LEFT"..timeLeft.."_DETAIL"];
                button.itemLink = link;
				button.PredictedTimeLeft:SetText(BMT_COMING_SOON);
				button.Warning:SetText(BMT_COMING_SOON);
                button:Show();
            else
                button:Hide();
			end
		else
			button:Hide();
		end
	end
	
	local totalHeight = numItems * scrollFrame.buttonHeight;
    local displayedHeight = numButtons * scrollFrame.buttonHeight;
    HybridScrollFrame_Update(scrollFrame, totalHeight, displayedHeight);
end

function BMT_SetNewUpdater(updaterName, updaterClass, updaterGuild, updateTime, updateDate)
	if (not BlackMarketTrackerMisc) then
		BlackMarketTrackerMisc = {};
	else
		-- throttle
		local currHour, currMinute = GetGameTime();
		local currTime = 60*currHour+currMinute;
		if(currTime == BlackMarketTrackerMisc.UpdateTime) then return end
	end
	
	BlackMarketTrackerMisc.UpdaterName = updaterName;
	BlackMarketTrackerMisc.UpdaterClass = updaterClass;
	BlackMarketTrackerMisc.UpdaterGuild = updaterGuild;
	BlackMarketTrackerMisc.UpdateTime = updateTime;
	BlackMarketTrackerMisc.UpdateDate = updateDate;
	
	-- if tracker is opened already, update the bottom text
	if(BlackMarketTrackerFrame:IsShown()) then
		BMTFadeState = 2;
		BlackMarketTrackerBottomText_FadingHandler();
	end
end

function BMT_AddNewToCurrentDB(name, texture, quantity, itemType,  level, levelType, sellerName, minBid, minIncrement, currBid, youHaveHighBid, numBids, timeLeft, link, marketID)
	-- check if table doesn't exist should already be done
	local currDBSize = #BlackMarketTrackerCurrentDB;
	local currHour, currMinute = GetGameTime();
	local currTime = 60*currHour+currMinute;
	BlackMarketTrackerCurrentDB[currDBSize+1] = {};
	BlackMarketTrackerCurrentDB[currDBSize+1].upateTime = currTime;
	BlackMarketTrackerCurrentDB[currDBSize+1].name = name;
	BlackMarketTrackerCurrentDB[currDBSize+1].texture = texture;
	BlackMarketTrackerCurrentDB[currDBSize+1].quantity = quantity;
	BlackMarketTrackerCurrentDB[currDBSize+1].itemType = itemType;
	BlackMarketTrackerCurrentDB[currDBSize+1].level = level;
	BlackMarketTrackerCurrentDB[currDBSize+1].levelType = levelType;
	BlackMarketTrackerCurrentDB[currDBSize+1].sellerName = sellerName;
	BlackMarketTrackerCurrentDB[currDBSize+1].minBid = minBid;
	BlackMarketTrackerCurrentDB[currDBSize+1].minIncrement = minIncrement;
	BlackMarketTrackerCurrentDB[currDBSize+1].currBid = currBid;
	-- TODO: Use for highest bidder detection
	BlackMarketTrackerCurrentDB[currDBSize+1].youHaveHighBid = youHaveHighBid;
	-- TODO: Check field
	BlackMarketTrackerCurrentDB[currDBSize+1].numBids = numBids;
	BlackMarketTrackerCurrentDB[currDBSize+1].timeLeft = timeLeft;
	-- completed
	if(timeLeft == 0) then
		BlackMarketTrackerCurrentDB[currDBSize+1].predictedLow = 0;
		BlackMarketTrackerCurrentDB[currDBSize+1].predictedHigh = 0;
	else
		BlackMarketTrackerCurrentDB[currDBSize+1].predictedLow = BMT_TIME_LEFT[timeLeft];
		BlackMarketTrackerCurrentDB[currDBSize+1].predictedHigh = BMT_TIME_LEFT[timeLeft+1];
	end
	BlackMarketTrackerCurrentDB[currDBSize+1].link = link;
	BlackMarketTrackerCurrentDB[currDBSize+1].marketID = marketID;
end

function BMT_UpdateExistingCurrentDBEntry(index, name, texture, quantity, itemType, level, levelType, sellerName, minBid, minIncrement, currBid, youHaveHighBid, numBids, timeLeft, link, marketID)
	local currHour, currMinute = GetGameTime();
	local currTime = 60*currHour+currMinute;
	-- throttle the updates
	if(currTime == BlackMarketTrackerCurrentDB[index].upateTime) then return end
	BlackMarketTrackerCurrentDB[index].upateTime = currTime;
	BlackMarketTrackerCurrentDB[index].name = name;
	BlackMarketTrackerCurrentDB[index].texture = texture;
	BlackMarketTrackerCurrentDB[index].quantity = quantity;
	BlackMarketTrackerCurrentDB[index].itemType = itemType;
	BlackMarketTrackerCurrentDB[index].level = level;
	BlackMarketTrackerCurrentDB[index].levelType = levelType;
	BlackMarketTrackerCurrentDB[index].sellerName = sellerName;
	BlackMarketTrackerCurrentDB[index].minBid = minBid;
	BlackMarketTrackerCurrentDB[index].minIncrement = minIncrement;
	BlackMarketTrackerCurrentDB[index].currBid = currBid;
	-- TODO: Use for highest bidder detection
	BlackMarketTrackerCurrentDB[index].youHaveHighBid = youHaveHighBid;
	-- TODO: Check field
	BlackMarketTrackerCurrentDB[index].numBids = numBids;
	BlackMarketTrackerCurrentDB[index].timeLeft = timeLeft;
	-- TODO: Recalculate predicted time
	if(timeLeft == 0) then
		BlackMarketTrackerCurrentDB[index].predictedLow = 0;
		BlackMarketTrackerCurrentDB[index].predictedHigh = 0;
	else
		BlackMarketTrackerCurrentDB[index].predictedLow = BMT_TIME_LEFT[timeLeft];
		BlackMarketTrackerCurrentDB[index].predictedHigh = BMT_TIME_LEFT[timeLeft+1];
	end
	BlackMarketTrackerCurrentDB[index].link = link;
	BlackMarketTrackerCurrentDB[index].marketID = marketID;
end

function BMT_GetFromDB(dbName, index)
	return dbName[index].name, dbName[index].texture, dbName[index].quantity, dbName[index].itemType, dbName[index].level, dbName[index].levelType, dbName[index].sellerName, dbName[index].minBid, dbName[index].minIncrement, dbName[index].currBid, dbName[index].youHaveHighBid, dbName[index].numBits, dbName[index].timeLeft, dbName[index].link, dbName[index].marketID;
end

function BMT_UpdateCurrentDB(name, texture, quantity, itemType, level, levelType, sellerName, minBid, minIncrement, currBid, youHaveHighBid, numBids, timeLeft, link, marketID)
	if(not BlackMarketTrackerCurrentDB) then
		BlackMarketTrackerCurrentDB = {};
	end
	local currDBSize = #BlackMarketTrackerCurrentDB;
	-- iterate through all fields, check for duplicates
	local duplicateIndex = 0;
	for i=1, currDBSize do
		if(BlackMarketTrackerCurrentDB[i].name == name) then
			duplicateIndex = i;
		end
	end
	-- new entry
	if(duplicateIndex == 0) then
		BMT_AddNewToCurrentDB(name, texture, quantity, itemType, level, levelType, sellerName, minBid, minIncrement, currBid, youHaveHighBid, numBids, timeLeft, link, marketID);
	-- existing entry
	else
		BMT_UpdateExistingCurrentDBEntry(duplicateIndex, name, texture, quantity, itemType, level, levelType, sellerName, minBid, minIncrement, currBid, youHaveHighBid, numBids, timeLeft, link, marketID);
	end
end

function BMT_MoveEntryToHistoryDB(index)
	if(not BlackMarketTrackerHistoryDB) then
		BlackMarketTrackerHistoryDB = {};
	end
	local histDBSize = #BlackMarketTrackerHistoryDB;
	BlackMarketTrackerHistoryDB[histDBSize+1] = {};
	BlackMarketTrackerHistoryDB[histDBSize+1].name = BlackMarketTrackerCurrentDB[index].name;
	BlackMarketTrackerHistoryDB[histDBSize+1].texture = BlackMarketTrackerCurrentDB[index].texture;
	BlackMarketTrackerHistoryDB[histDBSize+1].quantity = BlackMarketTrackerCurrentDB[index].quantity;
	BlackMarketTrackerHistoryDB[histDBSize+1].itemType = BlackMarketTrackerCurrentDB[index].itemType;
	BlackMarketTrackerHistoryDB[histDBSize+1].level = BlackMarketTrackerCurrentDB[index].level;
	BlackMarketTrackerHistoryDB[histDBSize+1].levelType = BlackMarketTrackerCurrentDB[index].levelType;
	BlackMarketTrackerHistoryDB[histDBSize+1].sellerName = BlackMarketTrackerCurrentDB[index].sellerName;
	BlackMarketTrackerHistoryDB[histDBSize+1].minBid = BlackMarketTrackerCurrentDB[index].minBid;
	BlackMarketTrackerHistoryDB[histDBSize+1].minIncrement = BlackMarketTrackerCurrentDB[index].minIncrement;
	BlackMarketTrackerHistoryDB[histDBSize+1].currBid = BlackMarketTrackerCurrentDB[index].currBid;
	BlackMarketTrackerHistoryDB[histDBSize+1].youHaveHighBid = BlackMarketTrackerCurrentDB[index].youHaveHighBid;
	BlackMarketTrackerHistoryDB[histDBSize+1].numBids = BlackMarketTrackerCurrentDB[index].numBids;
	BlackMarketTrackerHistoryDB[histDBSize+1].timeLeft = 0;
	BlackMarketTrackerHistoryDB[histDBSize+1].link = BlackMarketTrackerCurrentDB[index].link;
	BlackMarketTrackerHistoryDB[histDBSize+1].marketID = BlackMarketTrackerCurrentDB[index].marketID;
	table.remove(BlackMarketTrackerCurrentDB, index);
end

function BMT_NewSearchHandler()
	local playerName = UnitName("player");
	local _, playerClass = UnitClass("player");
	local playerGuild = GetGuildInfo("player");
	local updateHour, updateMinute = GetGameTime();
	local updateTime = 60*updateHour+updateMinute;
	local updateDate = date("%d/%m/%y");
	BMT_SetNewUpdater(playerName, playerClass, playerGuild, updateTime, updateDate);
	-- TODO: Friends
	SendAddonMessage("BMT", BMT_SYNC_SENDER_KEY..BMT_SEPARATOR..playerName..BMT_SEPARATOR..playerClass..BMT_SEPARATOR..playerGuild..BMT_SEPARATOR..updateTime..BMT_SEPARATOR..updateDate, "GUILD");
	local numItems = C_BlackMarket.GetNumItems();
	if(numItems == nil) then return end
	for i=1, numItems do
		local name, texture, quantity, itemType, _, level, levelType, sellerName, minBid, minIncrement, currBid, youHaveHighBid, numBids, timeLeft, link, marketID = C_BlackMarket.GetItemInfoByIndex(i);
		-- sometimes happens when item update event gets fired multiple times
		if(name ~= "") then
			BMT_UpdateCurrentDB(name, texture, quantity, itemType, level, levelType, sellerName, minBid, minIncrement, currBid, youHaveHighBid, numBids, timeLeft, link, marketID);
			-- sync
			-- TODO: Friends
			SendAddonMessage("BMT", BMT_SYNC_ITEM_KEY..BMT_SEPARATOR..name..BMT_SEPARATOR..texture..BMT_SEPARATOR..quantity..BMT_SEPARATOR..itemType..BMT_SEPARATOR..level..BMT_SEPARATOR..levelType..BMT_SEPARATOR..sellerName..BMT_SEPARATOR..minBid..BMT_SEPARATOR..minIncrement..BMT_SEPARATOR..currBid..BMT_SEPARATOR..tostring(youHaveHighBid)..BMT_SEPARATOR..numBids..BMT_SEPARATOR..timeLeft..BMT_SEPARATOR..link..BMT_SEPARATOR..marketID, "GUILD");
		end
	end
	-- move to history DB
	if(BlackMarketTrackerCurrentDB) then
		local currDBSize = #BlackMarketTrackerCurrentDB;
		-- iterate through all items in db
		local found;
		for i=1, currDBSize do
			found = 0;
			-- iterate through bmah items
			for j=1, numItems do
				local name = C_BlackMarket.GetItemInfoByIndex(j);
				-- avoid doing anything when C_BlackMarket.GetItemInfoByIndex returns an empty name
				if(BlackMarketTrackerCurrentDB[i].name == name or name == "") then
					found = 1;
				end
			end
			if(found == 0) then
				BMT_MoveEntryToHistoryDB(i);
			end
		end
	end
end

function BMT_Recalc(currRange, syncTimeDiff)
	-- new item
	--if(BMTPredictedLow == "99:99" or BMTPredictedHigh == "99:99") then
		if(currRange == 0) then
			BMT_Debug("auction already complete");
			return;
		elseif(currRange == 1) then
			BMTPredictedLow = "00:00";
			BMTPredictedHigh = string.format("%2.2d:%2.2d", 0, BMT_TIME_LEFT_SHORT);
		elseif(currRange == 2) then
			BMTPredictedLow = string.format("%2.2d:%2.2d", 0, BMT_TIME_LEFT_SHORT);
			BMTPredictedHigh = string.format("%2.2d:%2.2d", 0, BMT_TIME_LEFT_MEDIUM);
		elseif(currRange == 3) then
			BMTPredictedLow = string.format("%2.2d:%2.2d", 0, BMT_TIME_LEFT_MEDIUM);
			BMTPredictedHigh = string.format("%2.2d:%2.2d", 0, BMT_TIME_LEFT_HIGH);
		elseif(currRange == 4) then
			BMTPredictedLow = string.format("%2.2d:%2.2d", 0, BMT_TIME_LEFT_MEDIUM);
			BMTPredictedHigh = string.format("%2.2d:%2.2d", 99, 99);
		end
		BMT_Debug("BMTPredictedLow="..BMTPredictedLow.." BMTPredictedHigh="..BMTPredictedHigh);
	--end
end

function BMT_SyncSenderUpdateHandler(msg)
	local _, updaterName, updaterClass, updaterGuild, updateTime, updateDate = strsplit(BMT_SEPARATOR, msg);
	BMT_SetNewUpdater(updaterName, updaterClass, updaterGuild, updateTime, updateDate);
end

function BMT_SyncItemUpdateHandler(msg)
	local _, name, texture, quantity, itemType, level, levelType, sellerName, minBid, minIncrement, currBid, youHaveHighBid, numBids, timeLeft, link, marketID = strsplit(BMT_SEPARATOR, msg);
	local youHaveHighBidVal;
	if(youHaveHighBid == "true") then
		youHaveHighBidVal = true;
	else
		youHaveHighBidVal = false;
	end
	BMT_UpdateCurrentDB(name, texture, tonumber(quantity), itemType, tonumber(level), levelType, sellerName, tonumber(minBid), tonumber(minIncrement), tonumber(currBid), youHaveHighBidVal, tonumber(numBids), tonumber(timeLeft), link, tonumber(marketID));
end

function BMT_SyncVersionHandler(msg)
	local _, verNumberStr = strsplit(BMT_SEPARATOR, msg);
	local verNum = tonomber(verNumberStr);
	-- this is higher version
	if(BMTRevision > verNum) then
		SendAddonMessage("BMT", BMT_SYNC_VER_KEY..BMT_SEPARATOR..BMTRevision, "GUILD");
	elseif(BMTRevision < verNum) then
		BMTHighestRevision = verNum;
		-- if ui is open, update the bottom text
		if(BlackMarketTrackerFrame:IsShown()) then
			BMTFadeState = 0;
			BlackMarketTrackerBottomText_FadingHandler();
		end
	end
end

function BMT_AddonMsgReceiveHandler(prefix, msg, channel, sender)
	if(prefix ~= "BMT" or sender == UnitName("player")) then return end
	--BMT_Debug("msg="..msg);
	local key = strsplit(BMT_SEPARATOR, msg);
	if(key == BMT_SYNC_SENDER_KEY) then
		BMT_SyncSenderUpdateHandler(msg);
	elseif(key == BMT_SYNC_ITEM_KEY) then
		BMT_SyncItemUpdateHandler(msg);
	elseif(key == BMT_SYNC_VER_KEY) then
		BMT_SyncVersionHandler(msg);
	end
end

function BMT_OnLoginHandler()
	SendAddonMessage("BMT", BMT_SYNC_VER_KEY..BMT_SEPARATOR..BMTRevision, "GUILD");
	-- TODO: Sync items
end